@@ -20,6 +20,7 @@
 KSingleTabWidget::KSingleTabWidget( QWidget *parent, const char *name ) : QWidget( parent,name )
 {
 	layout = new QGridLayout( this, 1, 8 );
+	layout->setAlignment( Qt::AlignHCenter );
 	layout->setSpacing( 10 );
 	layout->setMargin( 15 );
 	layout->setColStretch( 7, 1 );
@@ -31,3 +32,12 @@ KSingleTabWidget::KSingleTabWidget( QWidget *parent, const char *name ) : QWidge
 KSingleTabWidget::~KSingleTabWidget()
 {
 }
+
+
+/*!
+    \fn KSingleTabWidget::getLayout()
+ */
+QGridLayout* KSingleTabWidget::getLayout()
+{
+	return layout;
+}
