@@ -30,6 +30,8 @@ class KSingleTabWidget : public QWidget
 public:
 	KSingleTabWidget( QWidget *parent=0, const char *name=0 );
 	~KSingleTabWidget();
+	QGridLayout* getLayout();
+protected:
 	QGridLayout* layout;
 };
 
