@@ -24,6 +24,7 @@ WCard::WCard( const char *device )
 	ethFD = socket(AF_INET, SOCK_DGRAM, 0);
 
 	this->device = new char[ 20 ];
+	this->dhcpcdPid = new char[ 20 ];
 
 	range = 0L;
 	stats = 0L;
@@ -42,6 +43,7 @@ WCard::WCard( const char *device )
 WCard::~WCard()
 {
 	delete device;
+	delete dhcpcdPid;
 	delete procBuffer;
 	delete stats;
 	delete range;
@@ -110,13 +112,16 @@ int WCard::getNetID()
 char* WCard::getIP()
 {
 	struct ifreq ifrq;
-	memset(&ifrq, 0, sizeof(struct ifreq));
-	strcpy(ifrq.ifr_name, device);
-	if( ioctl(ethFD, SIOCGIFADDR, &ifrq) < 0 )
+	memset( &ifrq, 0, sizeof( struct ifreq ) );
+	strcpy( ifrq.ifr_name, device );
+	if( ioctl( ethFD, SIOCGIFADDR, &ifrq ) < 0 )
+	{
 		return NULL;
-	struct sockaddr_in *addr;
-	addr = (sockaddr_in*)(&ifrq.ifr_addr);
-	return inet_ntoa(addr->sin_addr);
+	}
+// 	struct sockaddr_in *addr;
+// 	addr = (sockaddr_in*)(&ifrq.ifr_addr);
+// 	return inet_ntoa(addr->sin_addr);
+	return inet_ntoa( ((sockaddr_in*)(&ifrq.ifr_addr))->sin_addr );
 }
 
 
@@ -141,24 +146,31 @@ char* WCard::setIP( const char *ip )
 	
 	memcpy( &(ifrq.ifr_addr), addr, sizeof(struct sockaddr) );
 	
-	strcpy(ifrq.ifr_name, device);
-	if( ioctl(ethFD, SIOCSIFADDR, &ifrq) < 0 )
+	strcpy( ifrq.ifr_name, device );
+	if( ioctl( ethFD, SIOCSIFADDR, &ifrq ) < 0 )
+	{
 		return NULL;
+	}
+
+	if( setFlags( IFF_UP | IFF_RUNNING ) < 0 )
+	{
+		return NULL;
+	}
 	
 	return getIP();
 }
 
 
 /*!
-	\fn WCard::getProtocol(char *protocol)
+	\fn WCard::getProtocol( char *protocol )
  */
-int WCard::getProtocol(char *protocol)
+int WCard::getProtocol( char *protocol )
 {
 	if( iwIoctl( SIOCGIWNAME ) < 0)
 	{
 		return -1;
 	}
-	strncpy(protocol , wrq.u.name, strlen(wrq.u.name)+1 );
+	strncpy( protocol, wrq.u.name, strlen( wrq.u.name ) + 1 );
 	return 0;
 }
 
@@ -170,7 +182,7 @@ int WCard::getMode()
 {
 	if( iwIoctl( SIOCGIWMODE ) < 0 )
 		return -1;
-	return wrq.u.mode;  
+	return wrq.u.mode;
 }
 
 
@@ -828,7 +840,6 @@ APInfos WCard::scan( int & error, int & errorNo )
 				
 			}while( toSmall );
 		}
-	
 	}while( ! ok );
 	
 	if( wrq.u.data.length )
@@ -1085,13 +1096,12 @@ int WCard::frequency2Channel( double frequency, const struct iw_range* range )
  */
 bool WCard::getStats( iw_discarded& discarded, int& missedBeacon )
 {
-	discarded.nwid = 10;
-	discarded.code = 10;
-	discarded.fragment = 10;
-	discarded.retries = 10;
-	discarded.misc = 10;
-	missedBeacon = 10;
-
+	discarded.nwid = 0;
+	discarded.code = 0;
+	discarded.fragment = 0;
+	discarded.retries = 0;
+	discarded.misc = 0;
+	missedBeacon = 0;
 
 	ifstream procFile( "/proc/net/wireless" );
 	if( ! procFile.good() )
@@ -1118,14 +1128,13 @@ bool WCard::getStats( iw_discarded& discarded, int& missedBeacon )
 		deviceStream << foundStart;
 		
 		char buffer[ 32 ];
-		int dummy;
-		
+
 		// status
 		deviceStream >> buffer;
 		// 3 times quality
-		deviceStream >> dummy;
-		deviceStream >> dummy;
-		deviceStream >> dummy;
+		deviceStream >> buffer;
+		deviceStream >> buffer;
+		deviceStream >> buffer;
 		
 		deviceStream >> discarded.nwid;
 		deviceStream >> discarded.code;
@@ -1420,3 +1429,119 @@ int WCard::setEncodingMode( int mode )
 	return iwIoctl( SIOCSIWENCODE );
 }
 
+
+/*!
+    \fn WCard::isDhcpRunning()
+ */
+bool WCard::isDhcpRunning()
+{
+	char filename[ strlen( "/var/run/dhcpcd-" ) + strlen( device ) + strlen( ".pid" ) + 1 ];
+	sprintf( filename, "/var/run/dhcpcd-%s.pid", device );
+	ifstream file;
+	file.open( filename );
+	if( !file.is_open() )
+	{
+		return false;
+	}
+	file.close();
+	return true;
+}
+
+
+/*!
+    \fn WCard::startDhcpcd()
+ */
+std::string WCard::startDhcpcd()
+{
+	char cmd[256 ];
+	sprintf( cmd, "dhcpcd %s 2>&1", device );
+	FILE* pipe = popen( cmd, "r" );
+	if( ( pipe == NULL ) || ( (int)pipe == -1 ) )
+	{
+		return NULL;
+	}
+	int BUFFER_MAX = 256;
+	char buffer[BUFFER_MAX];
+	string s;
+	while( fgets( buffer, BUFFER_MAX, pipe ) != NULL )
+	{
+		s += buffer;
+	}
+	pclose( pipe );
+	return s;
+}
+
+
+/*!
+    \fn WCard::getDhcpcdPid()
+ */
+char* WCard::getDhcpcdPid()
+{
+	char filename[ strlen( "/var/run/dhcpcd-") + strlen( device ) + strlen( ".pid" ) + 1 ];
+	sprintf( filename, "/var/run/dhcpcd-%s.pid", device );
+	std::ifstream fin;
+	fin.open( filename );
+	if( !fin.is_open() )
+	{
+		return NULL;
+	}
+	fin >> dhcpcdPid;
+	fin.close();
+	return dhcpcdPid;
+}
+
+
+/*!
+    \fn WCard::stopDhcpcd()
+ */
+int WCard::stopDhcpcd()
+{
+	getDhcpcdPid();
+	if( dhcpcdPid == NULL )
+	{
+		return -2;
+	}
+	pid_t pid = atoi( dhcpcdPid );
+	return kill( pid, 15 ); // SIG_TERM
+}
+
+
+/*!
+    \fn WCard::setFlags( int flags )
+ */
+int WCard::setFlags( int flags )
+{
+	struct ifreq ifrq;
+	memset( &ifrq, 0, sizeof( struct ifreq ) );
+	strcpy( ifrq.ifr_name, device );
+	
+	int result = ioctl( ethFD, SIOCGIFFLAGS, &ifrq );
+	if( result < 0 )
+	{
+		return result;
+	}
+	
+	memset( &ifrq, 0, sizeof( struct ifreq ) );
+	strcpy( ifrq.ifr_name, device );
+	ifrq.ifr_flags |= flags;
+
+	return ioctl( ethFD, SIOCSIFFLAGS, &ifrq );
+}
+
+
+/*!
+    \fn WCard::getFlags()
+ */
+int WCard::getFlags()
+{
+	struct ifreq ifrq;
+	memset( &ifrq, 0, sizeof( struct ifreq ) );
+	strcpy( ifrq.ifr_name, device );
+	
+	int result = ioctl( ethFD, SIOCGIFFLAGS, &ifrq );
+	if( result < 0 )
+	{
+		return result;
+	}
+	return ifrq.ifr_flags;
+}
