@@ -25,37 +25,36 @@ WWidget::WWidget( WCard *card, QString cmd, QWidget *parent, const char *name )
 	this->card = card;
 	this->cmd = cmd;
 
-	buffer =  new char[ 1024 ];
+	buffer = new char[ 1024 ];
 
 	// i'm optimist
 	isWave = true;
 	
 	initTabs();
 	adjustEntries();
-	
+
 	scanThread = new QScanThread( card );
 	scanning = false;
-	
-	connectSignals();
 
-	setStaticInfo();
+	dhcpTimer = new QTimer( this );
+	dhcpThread = new QDhcpThread( card );
+	dhcpThreadRunning = false;
 	
-	this->setMainWidget( tab );
-
 	gateway = "";
 	
-	QToolButton* whatsThisButton = QWhatsThis::whatsThisButton ( this );
+	QToolButton* whatsThisButton = QWhatsThis::whatsThisButton( this );
 	KGuiItem whatsItem( i18n( "What's this?" ), whatsThisButton->iconSet(), i18n( "Show extended Tooltips" ) );
 	setButtonGuiItem( KDialogBase::User1, whatsItem );
-	connect( this, SIGNAL( user1Clicked() ), this, SLOT( whatsThisSlot() ) );
 	delete whatsThisButton;
 
 	KGuiItem rootItem( i18n( "Restart as root" ), KGlobal::iconLoader()->loadIcon( "launch", KIcon::Small ), i18n( "Restart the application with root privileges" ), QWhatsThisStrings::root() );
 	setButtonGuiItem( KDialogBase::User2, rootItem );
 	// show only if user isn't root
-	showButton( KDialogBase::User2, ( strcmp( getpwuid( getuid() )->pw_name, "root" ) != 0 ) );
-	connect( this, SIGNAL( user2Clicked() ), this, SLOT( rootSlot() ) );
-	
+	showButton( KDialogBase::User2, strcmp( getpwuid( getuid() )->pw_name, "root" ) );
+
+	connectSignals();
+	setStaticInfo();
+
 	setWhatsThis();
 }
 
@@ -65,38 +64,9 @@ WWidget::WWidget( WCard *card, QString cmd, QWidget *parent, const char *name )
 */
 WWidget::~WWidget()
 {
-	delete firstTab;
-	delete infoTab;
-	delete trafficTab;
-	delete deviceSelection;
-	delete IPEdit;
-	delete gatewayEdit;
-	delete nickEdit;
-	delete modeSelection;
-	delete linkBar;
-	delete signalBar;
-	delete noiseBar;
-	delete encodingLabel;
-	delete pollingSpin;
-	delete protocolLabel;
-	delete bitrateSelection;
-	delete channelSelection;
-	delete sensitivityLabel;
-	delete essidEdit;
-	delete macAPLabel;
-	delete powerEnabledSelection;
-	delete powerModeSelection;
-	delete powerGroup;
-	delete txPowerGroup;
-	delete authorLabel;
-	delete homepageLabel;
-	delete versionLabel;
-	delete licenseLabel;
-	delete emailLabel;
-	delete weVersionLabel;
-	delete scanThread;
-	delete scanWidget;
 	delete [] buffer;
+	delete scanThread;
+	delete dhcpThread;
 }
 
 
@@ -122,6 +92,7 @@ void WWidget::initTabs()
 	tab->addTab( infoTab, i18n( "Info" ) );
 	
 	deviceSelection = new KWCEntrySelection( i18n( "Device" ), true, firstTab );
+	dhcpEntry = new KWCEntryDhcp( firstTab );
 	IPEdit = new KWCEntryEdit( i18n( "IP Address"), firstTab );
 	gatewayEdit = new KWCEntryEdit( i18n( "Gateway"), firstTab );
 
@@ -154,7 +125,9 @@ void WWidget::initTabs()
 	rtsGroup = new KThresholdGroup( i18n( "RTS threshold" ), experiencedTab->leftBox );
 	fragGroup = new KThresholdGroup( i18n( "Fragmentation threshold" ), experiencedTab->leftBox );
 	txPowerGroup = new KTxPowerGroup( i18n( "Tx Power" ), experiencedTab->rightBox  );
-	powerGroup = new KPowerGroup( i18n( "Power Management" ), experiencedTab->rightBox  );
+	powerGroup = new KPowerGroup( i18n( "Power Management" ), experiencedTab->rightBox );
+	experiencedTab->leftBox->setStretchFactor( new QWidget( experiencedTab->leftBox ), 1 );
+	experiencedTab->rightBox->setStretchFactor( new QWidget( experiencedTab->rightBox ), 1 );
 
 	experiencedTab->rightBox->adjustSize();
 	experiencedTab->adjustSize();
@@ -185,6 +158,8 @@ void WWidget::initTabs()
 	{
 		trafficTab->clear( recv, sent );
 	}
+
+	this->setMainWidget( tab );
 }
 
 
@@ -194,13 +169,14 @@ void WWidget::initTabs()
 void WWidget::adjustEntries()
 {
 	deviceSelection->move( 0, 0 );
+	dhcpEntry->move( 3, 0 );
 	IPEdit->move( 1, 0 );
 	gatewayEdit->move( 2, 0 );
-	essidEdit->move( 3, 0 );
-	nickEdit->move( 4, 0 );
-	modeSelection->move( 5, 0 );
-	bitrateSelection->move( 6, 0 );
-	channelSelection->move( 7 , 0);
+	essidEdit->move( 4, 0 );
+	nickEdit->move( 5, 0 );
+	modeSelection->move( 6, 0 );
+	bitrateSelection->move( 7, 0 );
+	channelSelection->move( 8 , 0);
 	
 	linkBar->move( 0, 1 );
 	signalBar->move( 1, 1 );
@@ -210,7 +186,7 @@ void WWidget::adjustEntries()
 	macAPLabel->move( 5, 1 );
 	sensitivityLabel->move( 6, 1 );
 
-	pollingSpin->move( 7, 1 );
+	pollingSpin->move( 8, 1 );
 
 	authorLabel->move( 0, 0 );
 	homepageLabel->move( 1, 0 );
@@ -245,6 +221,12 @@ void WWidget::connectSignals()
 	connect( encGroup, SIGNAL( changed( int, unsigned char*, int ) ), this, SLOT( encryptionSlot( int, unsigned char*, int ) ) );
 	connect( encGroup, SIGNAL( changed( int ) ), this, SLOT( encryptionSlot( int ) ) );
 	connect( encGroup, SIGNAL( modeChanged( int ) ), this, SLOT( encryptionModeSlot( int ) ) );
+
+	connect( dhcpEntry, SIGNAL( buttonClicked() ), this, SLOT( dhcpSlot() ) );
+	connect( dhcpTimer, SIGNAL( timeout() ), this, SLOT( dhcpSlot() ) );
+
+	connect( this, SIGNAL( user1Clicked() ), this, SLOT( whatsThisSlot() ) );
+	connect( this, SIGNAL( user2Clicked() ), this, SLOT( rootSlot() ) );
 }
 
 
@@ -536,6 +518,24 @@ void WWidget::secondTimerSlot()
 		scanComplete( scanThread->apInfos, scanThread->error, scanThread->errorNo );
 		scanWidget->setScanning( false );
 	}
+	
+	if( dhcpThreadRunning && dhcpThread->finished() )
+	{
+		dhcpThreadRunning = false;
+		std::string result = dhcpThread->getResult();
+		if( result == NULL )
+		{
+			KMessageBox::error( this, i18n( "Failed to run dhcpcd" ) );
+		}
+		else if( result.length() > 0 )
+		{
+			KMessageBox::detailedError( this, i18n( "Failed to run dhcpcd." ), QString( result ) );
+		}
+		dhcpEntry->enableButton( true );
+	}
+
+	dhcpEntry->setStartText( ! card->isDhcpRunning() );
+	adjustSize();
 }
 
 
@@ -853,8 +853,11 @@ void WWidget::getWirelessDevices( QString preset )
  */
 void WWidget::scanSlot()
 {
-	scanThread->start();
-	scanning = true;
+	if( ! scanning )
+	{
+		scanThread->start();
+		scanning = true;
+	}
 }
 
 
@@ -942,9 +945,9 @@ void WWidget::rootSlot()
 
 
 /*!
-	\fn WWidget::connectToAp( QString essid, double frequency, QString mode )
+	\fn WWidget::connectToAp( QString essid, double frequency, QString mode, bool enc, bool dhcp )
  */
-void WWidget::connectToAp( QString essid, double frequency, QString mode )
+void WWidget::connectToAp( QString essid, double frequency, QString mode, bool enc, bool dhcp )
 {
 	if( ! essid.isEmpty() )
 	{
@@ -966,6 +969,10 @@ void WWidget::connectToAp( QString essid, double frequency, QString mode )
 			}
 		}
 	}
+	if( dhcp )
+	{
+		dhcpTimer->start( 1000, true );
+	}
 }
 
 
@@ -995,6 +1002,21 @@ Profile WWidget::getProfile()
 		profile.keyIndex = -1;
 		profile.key = "";
 	}
+
+	QString device = card->getDevice();
+	QString file = QString( "/var/run/dhcpcd-" ) + device + QString( ".pid" );
+	std::fstream fin;
+	fin.open( file.ascii(), std::ios::in );
+	profile.dhcp = fin.is_open();
+	fin.close();
+	if( profile.dhcp )
+	{
+		QString dhcpMessage = "The device " + device + " seems to be configured by dhcpcd.\n";
+		dhcpMessage += "Do you want to save the use of dhcpcd in the profile?\n";
+		dhcpMessage += "(So the next time you load this profile dhcpcd will be run automatically, otherwise the settings will be restored statically.)";
+		profile.dhcp = ( KMessageBox::questionYesNo( this, dhcpMessage, "save use of dhcp" ) == KMessageBox::Yes );
+	}
+	
 	return profile;
 }
 
@@ -1033,8 +1055,18 @@ void WWidget::setProfile( Profile profile )
 	{
 		KMessageBox::detailedError( this, i18n( "Failed to set channel." ), strerror( errno ) );
 	}
-	IPChangedSlot( profile.ip );
-	gatewaySlot( profile.gateway );
+
+	if( ! profile.dhcp )
+	{
+		IPChangedSlot( profile.ip );
+		gatewaySlot( profile.gateway );
+	}
+	else
+	{
+		// let the device settle down a few seconds
+		dhcpTimer->start( 1000, true );
+	}
+
 	modeSlot( profile.mode );
 
 	if( profile.security != IW_ENCODE_DISABLED )
@@ -1207,3 +1239,36 @@ void WWidget::encryptionModeSlot( int mode )
 
 	encGroup->setKeys( card->getKeys() );
 }
+
+
+/*!
+    \fn WWidget::getScanWidget()
+ */
+KScanWidget* WWidget::getScanWidget()
+{
+	return scanWidget;
+}
+
+
+/*!
+    \fn WWidget::dhcpSlot()
+ */
+void WWidget::dhcpSlot()
+{
+	if( !card->isDhcpRunning() )
+	{
+		if( !dhcpThreadRunning && !dhcpThread->running() )
+		{
+			dhcpThreadRunning = true;
+			dhcpThread->start();
+			dhcpEntry->enableButton( false );
+		}
+	}
+	else
+	{
+		if( card->stopDhcpcd() != 0 )
+		{
+			KMessageBox::detailedError( this, i18n( "Failed to stop dhcpcd." ), strerror( errno ) );
+		}
+	}
+}
